@@ -7,7 +7,8 @@ from trexanalytics.conf import UPSTREAM_UPDATED_DATETIME_FIELD_NAME, MERCHANT_DA
 import uuid, logging
 from trexmodel.models.datastore.analytic_models import UpstreamData
 from trexanalytics.bigquery_table_template_config import REGISTERED_CUSTOMER_TEMPLATE, REGISTERED_MERCHANT_TEMPLATE, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE,\
-    CUSTOMER_TRANSACTION_TEMPLATE, MERCHANT_CUSTOMER_REWARD_TEMPLATE 
+    CUSTOMER_TRANSACTION_TEMPLATE, MERCHANT_CUSTOMER_REWARD_TEMPLATE,\
+    MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE 
 from trexlib.utils.google.bigquery_util import default_serializable
 from datetime import datetime, timedelta
 from trexmodel.models.datastore.transaction_models import CustomerTransactionWithRewardDetails
@@ -75,6 +76,19 @@ __CUSTOMER_REWARD_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
                                             "RevertedDateTime"      : 'reverted_datetime',
                                             }
 
+__CUSTOMER_REDEMPTION_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
+                                            "CustomerKey"           : 'redeemed_customer_key',
+                                            "MerchantKey"           : 'redeemed_merchant_acct_key',
+                                            "RedeemOutlet"          : 'redeemed_outlet_key',
+                                            "TransactionId"         : 'transaction_id',
+                                            "RedeemAmount"          : 'redeem_amount',
+                                            "RewardFormat"          : 'reward_format',
+                                            "VoucherKeysList"       : 'redeemed_voucher_keys_list',
+                                            "RedeemedDateTime"      : 'redeemed_datetime',
+                                            "Reverted"              : 'is_revert',
+                                            "RevertedDateTime"      : 'reverted_datetime',
+                                            }
+
 
 upstream_schema_config = {
                             REGISTERED_MERCHANT_TEMPLATE            : __REGISTERED_MERCHANT_TEMPLATE_UPSTREAM_SCHEMA,
@@ -82,25 +96,30 @@ upstream_schema_config = {
                             MERCHANT_REGISTERED_CUSTOMER_TEMPLATE   : __MERCHANT_REGISTERED_CUSTOMER_TEMPLATE_UPSTREAM_SCHEMA,
                             CUSTOMER_TRANSACTION_TEMPLATE           : __CUSTOMER_TRANSACTION_DATA_TEMPLATE_UPSTREAM_SCHEMA,
                             MERCHANT_CUSTOMER_REWARD_TEMPLATE       : __CUSTOMER_REWARD_DATA_TEMPLATE_UPSTREAM_SCHEMA,
+                            MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE   : __CUSTOMER_REDEMPTION_DATA_TEMPLATE_UPSTREAM_SCHEMA,
                             }
 
 logger = logging.getLogger('upstream')
 
-def __create_upstream(upstream_entity, upstream_template, dataset_name, table_name, update_datetime=None, **kwargs):
+def __create_upstream(upstream_entity, upstream_template, dataset_name, table_name, streamed_datetime=None, **kwargs):
     upstream_json = {}
     if upstream_entity:
         schema = upstream_schema_config.get(upstream_template)
         for upstrem_field_name, attr_name in schema.items():
             upstream_json[upstrem_field_name] = default_serializable(getattr(upstream_entity, attr_name))
     
-    if update_datetime is None:
-        update_datetime = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+    if streamed_datetime is None:
+        streamed_datetime = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
             
     upstream_json['Key'] = uuid.uuid1().hex
-    upstream_json[UPSTREAM_UPDATED_DATETIME_FIELD_NAME] = default_serializable(update_datetime)
+    upstream_json[UPSTREAM_UPDATED_DATETIME_FIELD_NAME] = default_serializable(streamed_datetime)
+    
+    #year = update_datetime.yeaar
+    
+    #dataset_with_year_prefix = '{year}_{dataset}'.format(year=year, dataset=dataset_name)
     
     for key, value in kwargs.items():
-        upstream_json[key] = convert_to_serializable_value(value)
+        upstream_json[key] = convert_to_serializable_value(value, datetime_format='%Y-%m-%d %H:%M:%S', date_format='%Y-%m-%d', time_format='%H:%M:%S')
     
     logger.debug('-------------------------------------------------')
     logger.debug('upstream_json=%s', upstream_json)
@@ -109,75 +128,94 @@ def __create_upstream(upstream_entity, upstream_template, dataset_name, table_na
     
 
 def create_registered_customer_upstream_for_system(customer):
-    update_datetime     = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+    streamed_datetime   = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
     
     table_name          = REGISTERED_CUSTOMER_TEMPLATE
-    final_table_name    = '{}_{}'.format(table_name, update_datetime.strftime('%Y%m%d'))
+    final_table_name    = '{}_{}'.format(table_name, streamed_datetime.strftime('%Y%m%d'))
     
-    __create_upstream(customer, REGISTERED_CUSTOMER_TEMPLATE, SYSTEM_DATASET, final_table_name, update_datetime=update_datetime)
+    __create_upstream(customer, REGISTERED_CUSTOMER_TEMPLATE, SYSTEM_DATASET, final_table_name, streamed_datetime=streamed_datetime)
     
 def create_merchant_registered_customer_upstream_for_merchant(customer):
-    update_datetime     = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+    streamed_datetime   = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
     
     table_name          = MERCHANT_REGISTERED_CUSTOMER_TEMPLATE
     merchant_acct       = customer.registered_merchant_acct
     account_code        = merchant_acct.account_code.replace('-','')
-    final_table_name    = '{}_{}_{}'.format(table_name, account_code, update_datetime.strftime('%Y%m%d'))
+    final_table_name    = '{}_{}_{}'.format(table_name, account_code, streamed_datetime.strftime('%Y%m%d'))
     
-    __create_upstream(customer, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE, MERCHANT_DATASET, final_table_name, update_datetime=update_datetime)    
+    __create_upstream(customer, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE, MERCHANT_DATASET, final_table_name, streamed_datetime=streamed_datetime)    
     
-def create_merchant_customer_transaction_upstream_for_merchant(transaction_details, update_datetime=None):
-    if update_datetime is None:
-        update_datetime     = transaction_details.transact_datetime
+def create_merchant_customer_transaction_upstream_for_merchant(transaction_details, streamed_datetime=None):
+    if streamed_datetime is None:
+        streamed_datetime     = transaction_details.transact_datetime
         
     table_name          = CUSTOMER_TRANSACTION_TEMPLATE
     merchant_acct       = transaction_details.transact_merchant_acct
     account_code        = merchant_acct.account_code.replace('-','')
-    final_table_name    = '{}_{}_{}'.format(table_name, account_code, update_datetime.strftime('%Y%m%d'))
+    final_table_name    = '{}_{}_{}'.format(table_name, account_code, streamed_datetime.strftime('%Y%m%d'))
     
     __create_upstream(transaction_details, CUSTOMER_TRANSACTION_TEMPLATE, MERCHANT_DATASET, final_table_name, 
-                      update_datetime=update_datetime, Reverted=False, RevertedDateTime=None)
+                      streamed_datetime=streamed_datetime, Reverted=False, RevertedDateTime=None)
     
-def create_merchant_customer_transaction_reveted_upstream_for_merchant(transaction_details, reverted_datetime, update_datetime=None):
-    if update_datetime is None:
-        update_datetime     = transaction_details.transact_datetime
+def create_merchant_customer_transaction_reverted_upstream_for_merchant(transaction_details, reverted_datetime):
+    partition_datetime = transaction_details.transact_datetime
         
     table_name          = CUSTOMER_TRANSACTION_TEMPLATE
     merchant_acct       = transaction_details.transact_merchant_acct
     account_code        = merchant_acct.account_code.replace('-','')
-    final_table_name    = '{}_{}_{}'.format(table_name, account_code, update_datetime.strftime('%Y%m%d'))
+    final_table_name    = '{}_{}_{}'.format(table_name, account_code, partition_datetime.strftime('%Y%m%d'))
     
     __create_upstream(transaction_details, CUSTOMER_TRANSACTION_TEMPLATE, MERCHANT_DATASET, final_table_name, 
-                      update_datetime=update_datetime, Reverted=True, RevertedDateTime=reverted_datetime)    
+                      streamed_datetime=reverted_datetime, Reverted=True, RevertedDateTime=reverted_datetime)    
 
-def create_merchant_customer_reward_upstream_for_merchant(transaction_details, reward_details, update_datetime=None):
-    if update_datetime is None:
-        update_datetime     = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+def create_merchant_customer_reward_upstream_for_merchant(transaction_details, reward_details, streamed_datetime=None):
+    if streamed_datetime is None:
+        streamed_datetime     = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
     
     table_name          = MERCHANT_CUSTOMER_REWARD_TEMPLATE
     merchant_acct       = transaction_details.transact_merchant_acct
     account_code        = merchant_acct.account_code.replace('-','')
-    final_table_name    = '{}_{}_{}'.format(table_name, account_code, update_datetime.strftime('%Y%m%d'))
+    final_table_name    = '{}_{}_{}'.format(table_name, account_code, streamed_datetime.strftime('%Y%m%d'))
     
     transaction_details_with_reward_details = CustomerTransactionWithRewardDetails(transaction_details, reward_details)
     
     __create_upstream(transaction_details_with_reward_details, MERCHANT_CUSTOMER_REWARD_TEMPLATE, MERCHANT_DATASET, final_table_name, 
-                      update_datetime=update_datetime, Reverted=False, RevertedDateTime=None)
+                      streamed_datetime=streamed_datetime, Reverted=False, RevertedDateTime=None)
     
-def create_merchant_customer_reward_reverted_upstream_for_merchant(transaction_details, reward_details, reverted_datetime, update_datetime=None):
-    if update_datetime is None:
-        update_datetime     = transaction_details.transact_datetime
+def create_merchant_customer_reward_reverted_upstream_for_merchant(transaction_details, reward_details, reverted_datetime):
+    partition_datetime = transaction_details.transact_datetime
     
     
     table_name          = MERCHANT_CUSTOMER_REWARD_TEMPLATE
     merchant_acct       = transaction_details.transact_merchant_acct
     account_code        = merchant_acct.account_code.replace('-','')
-    final_table_name    = '{}_{}_{}'.format(table_name, account_code, update_datetime.strftime('%Y%m%d'))
+    final_table_name    = '{}_{}_{}'.format(table_name, account_code, partition_datetime.strftime('%Y%m%d'))
     
     transaction_details_with_reward_details = CustomerTransactionWithRewardDetails(transaction_details, reward_details)
     
     __create_upstream(transaction_details_with_reward_details, MERCHANT_CUSTOMER_REWARD_TEMPLATE, MERCHANT_DATASET, final_table_name, 
-                      update_datetime=update_datetime, Reverted=True, RevertedDateTime=reverted_datetime)    
-
+                      streamed_datetime=reverted_datetime, Reverted=True, RevertedDateTime=reverted_datetime)    
 
 
+def create_merchant_customer_redemption_upstream_for_merchant(redemption_details, streamed_datetime=None):
+    if streamed_datetime is None:
+        streamed_datetime     = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+    
+    table_name          = MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE
+    merchant_acct       = redemption_details.redeemed_merchant_acct
+    account_code        = merchant_acct.account_code.replace('-','')
+    final_table_name    = '{}_{}_{}'.format(table_name, account_code, streamed_datetime.strftime('%Y%m%d'))
+    
+    __create_upstream(redemption_details, MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE, MERCHANT_DATASET, final_table_name, 
+                      streamed_datetime=streamed_datetime, Reverted=False, RevertedDateTime=None)
+    
+def create_merchant_customer_redemption_reverted_upstream_for_merchant(redemption_details, reverted_datetime):
+    partition_datetime = redemption_details.redeemed_datetime
+    
+    table_name          = MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE
+    merchant_acct       = redemption_details.redeemed_merchant_acct
+    account_code        = merchant_acct.account_code.replace('-','')
+    final_table_name    = '{}_{}_{}'.format(table_name, account_code, partition_datetime.strftime('%Y%m%d'))
+    
+    __create_upstream(redemption_details, MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE, MERCHANT_DATASET, final_table_name, 
+                      streamed_datetime=reverted_datetime, Reverted=True, RevertedDateTime=reverted_datetime)    
